#!/usr/bin/env python

import skimage.restoration
from skimage import img_as_float
import numpy as np
from PIL import Image
import os
from tqdm import tqdm
import csv
import rawpy

imagesFolderPath = 'flat-field/nef'
imagesFolderPathFileName = imagesFolderPath.replace('/', '_')
denoiser = 'wavelet'
npArrayFilePath = f'mean_{imagesFolderPathFileName}_{denoiser}.npy'

denoise = getattr(skimage.restoration, f'denoise_{denoiser}')

mean = None
numberOfImagesInMean = 0

imagesFileNames = os.listdir(imagesFolderPath)

requiresRaiseFiltering = False
if requiresRaiseFiltering:
    files = {}

    with open('RAISE_all.csv') as csvfile:
        reader = csv.DictReader(csvfile)
        for row in tqdm(list(reader), 'CSV parsing'):
            file = row['File'] + '.png'
            files[file] = row

    imagesFileNames = [imageFileName for imageFileName in tqdm(imagesFileNames, 'Filtering images') if files[imageFileName]['Device'] == 'Nikon D7000' and Image.open(f'{imagesFolderPath}/{imageFileName}').size == (4946, 3278)]

minGreen = 308#None

def treatImage(imageFileName):
    global mean, numberOfImagesInMean, minGreen
    imageFilePath = f'{imagesFolderPath}/{imageFileName}'
    if imageFileName.endswith('.NEF'):
        with rawpy.imread(imageFilePath) as raw:
            colorDesc = raw.color_desc.decode('ascii')
            assert colorDesc == 'RGBG'
            assert np.array_equal(raw.raw_pattern, np.array([[0, 1], [3, 2]], dtype = np.uint8))
            rawImageVisible = raw.raw_image_visible.copy()
            greenRawImageVisible = rawImageVisible[1::2, ::2]
            '''
            greenRawImageVisibleMin = greenRawImageVisible.min()
            if minGreen is None or greenRawImageVisibleMin < minGreen:
                minGreen = greenRawImageVisibleMin
                print(minGreen)
            '''
            imageNpArray = (greenRawImageVisible - minGreen) / (4908 - minGreen)
            # Pay attention to range of values expected by the denoiser.
            # Indeed if provide the thousands valued raw image, then the denoiser only returns values between 0 and 1 and making the difference between both look pointless.
            #exit(1)
    else:
        imagePil = Image.open(imageFilePath)
        imageNpArray = img_as_float(np.array(imagePil))
    # As the arguments differ from a denoiser to the other should use a match instead.
    imageDenoisedNpArray = denoise(imageNpArray, rescale_sigma=True)
    #print(imageNpArray, imageDenoisedNpArray)
    imageNoiseNpArray = imageNpArray - imageDenoisedNpArray
    if mean is None:
        mean = imageNoiseNpArray
    else:
        mean = ((mean * numberOfImagesInMean) + imageNoiseNpArray) / (numberOfImagesInMean + 1)
    numberOfImagesInMean += 1

for imageFileName in tqdm(imagesFileNames, 'Denoising images'):
    treatImage(imageFileName)

with open(npArrayFilePath, 'wb') as f:
    np.save(f, mean)
